;
; Julia Set explorer program for the CX16, written in Prog8.
;
;  by SMY 12/22
;
; This program is a descendant of a program written for the C64 DTV and a cousin to a program
; that I wrote in assembly last year. It's slow, even with 'warp mode' in the emulator.
; Not recomended for 'real' hardware...unless you are immortal!
; Runs best with sdcard image attached!

%import floats
%import syslib
%import conv
%import math
%import graphics
%import palette
%import string
%import psg
%zeropage basicsafe

; Note: this program is compatible with Commander X16 only.


main {

str base =  "        "
str wfilenam = "@0:yyyymmdd.z,s,w"
str rfilenam = "@0:yyyymmdd.z,s,r"
str mandfnr = "@0:20000101.0,s,r"
str mandfnw = "@0:20000101.0,s,w"
str depth = "0123456789abcdefghijklmnopqrstuvwxyz"

; Prog8 limits float arrays to ~50 elements max
float[40] index1
float[40] index2
float[40] index3
float[40] index4
float[40] index5
float[40] index6
float[40] index7
float[40] index8

;Sound definitions

ubyte SQUARE 	= %00111111		;pulse width @ maximum
ubyte TRIANGLE	= %10000000
ubyte NOISE		= %11000000
ubyte SAWTOOTH	= %01000000

ubyte VOLUME	= %11111111		;left & right, maximum

;
uword VOICE0 	= $F9C0
uword VOICE1	= $F9C4 
uword VOICE2	= $F9C8

uword FSHARP	= $01F1
uword GSHARP	= $020D
uword E			= $01BA
uword B			= $0297

ubyte key
ubyte imax = 16 
ubyte o_flag = 0		;optimize flag toggle
ubyte m_flag = 0		;mouse flag, 0=off, 1=on
ubyte button
ubyte debug = 0

;Mandelbrot/Julia Set coordinates
float posz = 0.0			;this is a waste variable to keep posx from getting overwritten by set_color()
float posx = -0.5			;I haven't figured out the correct solution yet
float posy = 0.0
float tx
float ty
float cIM
float cRE
float oldx
float oldy
float zm = 0.875
float i_limit = 4.0

float[40] stx = 0.0
float[40] sty = 0.0
float[40] sIM = 0.0
float[40] sRE = 0.0
float[40] szm = 0.875
ubyte[40] simax = 16

; x & y pixel increment values
float mx = 10.66667
float my = 240/2/10

;cursor position variables
uword cx = 160
uword cy = 120

float fx
float fy

; variables to track saved screen history. (not implemented yet)
ubyte history = 0
ubyte current = 0
ubyte in_julia = 0


const uword width = 320
const ubyte height = 240

					
;custom palette								
uword[] pal_a = [	$0000, $0fff, $0800, $0afe, $0c4c, $00c5, $000a, $0ee7, $0d85, $0640, $0f77, $0af6, $008f, $0bbb, $0eee, $0fff,
					$0000, $0111, $0222, $0333, $0444, $0555, $0666, $0777, $0888, $0999, $0aaa, $0bbb, $0ccc, $0ddd, $0eee, $0fff, 	
					$0211, $0433, $0644, $0866, $0a88, $0c99, $0fbb, $0211, $0422, $0633, $0844, $0a55, $0c66, $0f77, $0200, $0411,	
					$0611, $0822, $0a22, $0c33, $0f33, $0200, $0400, $0600, $0800, $0a00, $0c00, $0f00, $0221, $0443, $0664, $0886, 
 					$0c02, $0c12, $0c22, $0c32, $0c42, $0c52, $0c62, $0c72, $0c82, $0c92, $0ca2, $0cb2, $0cc2, $0cd2, $0ce2, $0cf2,
					$0fc3, $0210, $0430, $0640, $0860, $0a80, $0c90, $0fb0, $0121, $0343, $0564, $0786, $09a8, $0bc9, $0dfb, $0121,
 					$000a, $0019, $0028, $0037, $0046, $0055, $0064, $0073, $0082, $0091, $00a0, $00b1, $00c2, $00d3, $00e4, $00f5,
					$0480, $05a0, $06c0, $07f0, $0121, $0343, $0465, $0686, $08a8, $09ca, $0bfc, $0121, $0242, $0364, $0485, $05a6 ]	
					
uword[] pal_b = [	$06c8, $07f9, $0020, $0141, $0162, $0283, $02a4, $03c5, $03f6, $0020, $0041, $0061, $0082, $00a2, $00c3, $00f3, 	
					$0122, $0344, $0466, $0688, $08aa, $09cc, $0bff, $0122, $0244, $0366, $0488, $05aa, $06cc, $07ff, $0022, $0144, 	
					$0166, $0288, $02aa, $03cc, $03ff, $0022, $0044, $0066, $0088, $00aa, $00cc, $00ff, $0112, $0334, $0456, $0668,
					$088a, $09ac, $0bcf, $0112, $0224, $0346, $0458, $056a, $068c, $079f, $0002, $0114, $0126, $0238, $024a, $035c, 	
					$036f, $0002, $0014, $0016, $0028, $002a, $003c, $003f, $0112, $0334, $0546, $0768, $098a, $0b9c, $0dbf, $0112,	
					$0324, $0436, $0648, $085a, $096c, $0b7f, $0102, $0214, $0416, $0528, $062a, $083c, $093f, $0102, $0204, $0306,	
					$0408, $050a, $060c, $070f, $0212, $0434, $0646, $0868, $0a8a, $0c9c, $0fbe, $0211, $0423, $0635, $0847, $0a59,   
					$0c6b, $0f7d, $0201, $0413, $0615, $0826, $0a28, $0c3a, $0f3c, $0201, $0403, $0604, $0806, $0a08, $0c09, $0f0b ]			
; used for rotating palette
uword[16] pal_c

ubyte[] CROSSHAIR = [ 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 
					0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 0, 
					0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 0, 0, 
					0, 0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 1, 2, 0, 0, 0, 
					0, 0, 0, 0, 2, 1, 0, 0, 0, 0, 1, 2, 0, 0, 0, 0,  
					0, 0, 0, 0, 0, 2, 1, 1, 1, 1, 2, 0, 0, 0, 0, 0,
					0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 
					0, 0, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 0, 
					0, 0, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 0,  
					0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 
					0, 0, 0, 0, 0, 2, 1, 1, 1, 1, 2, 0, 0, 0, 0, 0, 
					0, 0, 0, 0, 2, 1, 0, 0, 0, 0, 1, 2, 0, 0, 0, 0,
					0, 0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 1, 2, 0, 0, 0,
					0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 0, 0,
					0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 0,
					2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2 ] 

; used for copying data to/from vram
ubyte[160] rowbuf_a
ubyte[160] rowbuf_b
ubyte[160] rowbuf_c
ubyte[160] rowbuf_d 


	sub start()  {
		float result
		ubyte st
		float ocx
		float ocy
		float dx
		float dy
		ubyte h_flag = 0	
		
			simax[0] = imax
			stx[0] = -0.500
			sty[0] = 0.0
			sRE[0] = 0.0
			sIM[0] = 0
			szm[0] = zm
			
			graphics.enable_bitmap_mode()
			graphics.clear_screen(9, 0)
			
			def_sprite(1,&CROSSHAIR, $e000, 1, true)

			palette.set_rgb(&pal_a, 256)
										
			do_indexx(0.875,posx)	
			
			
			
			if check_file(mandfnr) != 0 {
				mandelbrot(0.875,posy,16)								
				write_file(mandfnw) }						
			else {						
				read_mand() }						
							
	        zero_buff(&rowbuf_a,160)
	        zero_buff(&rowbuf_b,160)							
								
			ding_dong()		
										
			get_date()		
											
			tx = posx					
			ty = posy					
									
			cRE = posx
			cIM = posy
			
			oldx = posx
			oldy = posy
			
			fx = cx as float
			fy = cy as float
			
			ocx = fx
			ocy = fy
			
			current = 0
			
			status(current)
			cx16.mouse_config2(1)
			m_flag = 1	
			
			repeat {			;main loop
				key = 0			
			
				key = cbm.GETIN()	
				
				
				
				if m_flag {
					button = cx16.mouse_pos()
					if button & 1 == 1 {								
						cx = cx16.r0
						cy = cx16.r1
						cx16.sprite_set_position(cx - 8, cy - 8, 1)
						;%asm{{stp}}
						fx = cx as float
						fy = cy as float
						dx = fx - 160.0
						result = dx/mx
						result *= (1/zm)						
						result /= 10
						tx = posx+result 										
																			
						dy = fy-120
						result = dy/my
						result *= (1/zm)
						result /= 10
						ty = posy+result 
						
						stx[current] = tx
						sty[current] = ty
						;%asm{{stp}}
								
						status(current) 
						
						}
						
					  
					   
					}

;shouldn't need this and it there is 'weird' behavior too...mouse cursor positioning becomes 'erratic'					
;					button = cx16.mouse_pos()
;					if button {
;						m_flag = 0 }
;					else {
;						m_flag = 1 }
						
					when key {	
							
				  	61 -> { 					;+ Iterations
							if imax < 255 {
								imax = imax+16 }							
							simax[current] = imax														
							status(current)				
								}				
					17 -> { 					;cursor up
					
						result = 1/my
						result *= (1/zm)
						result /= 10
						ty = ty+result					
						sty[current] = ty				
						status(current) 
						cy++
						fy = cy as float
						cx16.sprite_set_position(cx - 8, cy - 8, 1) }	
					
					29 -> { 					;cursor right					
						result = 1/mx
						result *= (1/zm)
						result /= 10
						tx = tx+result						
						stx[current] = tx											
						status(current) 
						cx++
						fx = cx as float
						cx16.sprite_set_position(cx - 8, cy - 8, 1) }										
										
										
					44 -> {						; history reverse
						if current > 0 and history != 1 {			
							if h_flag == 0 {
								current--		
								h_flag = 1 }
							current--									
							sys.memcopy(&depth+current,&rfilenam+12,1)
							txt.plot(0,26)
							txt.print(rfilenam)
							read_file(rfilenam) 
							posx = stx[current]
							posy = sty[current]							
							zm = szm[current]			
							status(current)
						 }				
						else beep(0,SQUARE,B,VOLUME,20)					
											
							}		
									
					46 -> {						; history forward
						if current < history-1 and history !=0 {
							current++ 
							sys.memcopy(&depth+current,&rfilenam+12,1)						
							txt.plot(0,26)
							txt.print(rfilenam)					
							read_file(rfilenam) 
							posx = stx[current]
							posy = sty[current]							
							zm = szm[current]	
							status(current) 
							}						
						else beep(0,SQUARE,B,VOLUME,20)														
						}
				
				  	45 -> { 					;- Iterations
							if imax > 16 imax = imax-16			
							simax[current] = imax
							status(current)			
								}											
																																						
					67 -> {						;color shift (byte value, will wrap)			
						roll_colors()		
							}														
							
					72 -> {						;home (redisplay base Mandelbrot)
						cx16.mouse_config2(0)						
						posx = cRE
						posy = cIM
						tx = posx					
						ty = posy							
														
							
						szm[current] = 0.875
						stx[current] = 0
						sty[current] = 0
						sIM[current] = cIM
						sRE[current] = cRE
						simax[current] = imax							
							
						
						zm = 0.875
;						szm[history] = 0.875
						do_indexx(zm,posx)	
						;imax = 16
						graphics.clear_screen(9, 0)
						txt.clear_screen()	
						read_mand()	
						fx = ocx
						fy = ocy
						in_julia = 0 
						status(current)
						cx16.mouse_config2(1)
						m_flag = 1						
						posx = -0.500
						posy = 0.0
						tx = posx
						ty = posy
						}
							
							 
				
					73 -> { 					; Zoom In
							zm = zm*2						
							szm[current] = zm		
							status(current) 
								}  	
										
										
					74 -> {						;Julia
							if in_julia == 0 {
								cRE = tx
								cIM = ty
								sIM[history] = cIM
								sRE[history] = cRE
								tx = 0.0
								ty = 0.0

								oldy = posy

								stx[current] = 0.0
								sty[current] = 0.0								
												
								posx = -0.0
								posy = 0.0
								posz = 0.0
								ocx = fx
								ocy = fy
								in_julia = 1 }
							else {

								stx[history] = tx
								sty[history] = ty								
								posx = stx[history]
								posy = sty[history]						
														
								sIM[history] = cIM
								sRE[history] = cRE														
														
								}
								cx16.sprite_set_position(8000, 0, 1)
								zero_buff(&rowbuf_a,160)
	        					zero_buff(&rowbuf_b,160)		
							szm[history] = zm
							simax[history] = imax
							cbm.SETTIM(0, 0, 0)
							graphics.clear_screen(9, 0)
							txt.clear_screen()								
							do_indexx(zm,posx)
							cx16.mouse_config2(0)						
							sys.memcopy(&depth+history,&wfilenam+12,1)																
							if debug == 1 txt.print(wfilenam)																									
							h_flag = 0																					
							julia(zm,imax,posx,posy)								
							history++
							current = history
							update_current()
								
							}				
					
					79 -> { 					;Zoom Out	
							zm = zm/2		
							szm[current] = zm
							status(current) 		
									}	
							
					80 -> {						;toggle speedup/cheat 
						if i_limit == 4.0 i_limit = 2.0
						else {
							i_limit = 4.0 }
						o_flag = not o_flag							
						status(current) 
									}
																	
					81 -> sys.reset_system() 	; Quit
					
					83 -> {
						if in_julia == 1 {
							status(current-1)	;status
							 }
						}
						
									 					
					86 -> { 					; View
						txt.clear_screen()
						cx16.mouse_config2(0)
						m_flag = 0
										}
										
										
					145 -> { 					;cursor down
						result = 1/my
						result *= (1/zm)
						result /= 10
						ty = ty-result							
						sty[current] = ty							
						status(current) 
						cy--
						fy = cy as float
						cx16.sprite_set_position(cx - 8, cy - 8, 1) }					
					
					157 ->  {					;cursor left
						result = 1/mx
						result *= (1/zm)
						result /= 10
						tx = tx-result
						stx[current] = tx					
						status(current) 
						cx--
						fx = cx as float
						cx16.sprite_set_position(cx - 8, cy - 8, 1)}																					
				}													
																										
		} 
		
		}			
		
sub update_current() {
	szm[current] = szm[current-1]
	stx[current] = stx[current-1]
	sty[current] = sty[current-1]
	sIM[current] = sIM[current-1]
	sRE[current] = sRE[current-1]
	simax[current] = simax[current-1]
	if debug == 1 {
		txt.plot(0, 10)
		txt.print_ub(current)
		txt.plot(0,11)
		floats.print_f(stx[current]) }
}
	
		
			
;shift palette arrays by 1 row of 16 colors
sub roll_colors() {
	ubyte key = 0
	
	do {
	
		cx16.memory_copy(&pal_a+32,&pal_c,32)
		cx16.memory_copy(&pal_a+64,&pal_a+32,448)	
		cx16.memory_copy(&pal_c,&pal_b+192,32)	
		
		sys.waitvsync()
	
		palette.set_rgb(&pal_a, 256)
		; the following code works best at normal speed (not warp mode in emulator)
		sys.wait(30)
		repeat 12000 {
			key = cbm.GETIN() 
		 	if key == 82 break }
		
		
	} until key == 82				;r key
			
}

	
sub status(ubyte cur) {

		if current < history-1 {
			cx16.mouse_config2(0)
			m_flag = 0 }
		else {
			cx16.mouse_config2(1)
			m_flag = 1 }	
				
		txt.color2(1,0)
			
		txt.clear_screen()
	
        txt.plot(0, 0)
        txt.print("x: ")
        floats.print_f(stx[cur])
						
        txt.plot(0, 1)
        txt.print("y: ")
        floats.print_f(sty[cur])					
	
        txt.plot(17, 0)
        txt.print("zoom:")
        txt.plot(21, 0)
        floats.print_f(szm[cur])				
				
        txt.plot(17, 1)
        txt.print("iter:")
        txt.print_ub(simax[cur])				
        
        txt.plot(35,0)
        txt.print("mode")

        
        if o_flag == 0 {
        	txt.plot(34,1)
        	txt.print("normal") }
        else {
            txt.plot(35,1)
        	txt.print("fast") }
        	
       if debug == 1{
       		txt.plot(0,28)
       		txt.print("next: ")
       		txt.print_ub(history)
       		txt.plot(30,28)
       		txt.print("current: ")
       		txt.print_ub(cur) }

		txt.plot(0,29)
		txt.print("cre:")
		floats.print_f(sRE[cur])    		
		
		txt.plot(20,29)
		txt.print("cim:")
		floats.print_f(sIM[cur]) 					
									  					
}

		
		
sub mandelbrot(float zoom, float ypos, ubyte max_iter) {
	float xsquared 
	float ysquared 
	float x 
	float y 
	float ty
	float xx
	float yy
	float r
	ubyte iter 
	ubyte ti
	ubyte i
    ubyte yz_flag
	uword pixelx
	ubyte pixely
	
			r = 1/my
			r *= (1/zm)
			r /= 10
			yz_flag = 0
			
			if ypos < 0 ty = -ypos
			else {
				ty = ypos }
	
		  	if ypos == 0 yz_flag = 1
			else if ty < r {
				yz_flag = 1 }	
		
			if history < 31 history++
			
			ti = max_iter
			
        	cbm.SETTIM(0, 0, 0)
					
			;plot mandelbrot set, every other column to 'speed up' ;-)
			for pixely in 0 to height-1 {
            	yy = (pixely as float - 120)
            	yy = yy/(zoom*120) + ypos
            	i = 0
            	for pixelx in 0 to 39 {
            		if pixelx & 1 {
                		xx = index1[i]

                		xsquared = 0.0
                		ysquared = 0.0
                		x = 0.0
                		y = 0.0
                		iter = 0

                		while iter<ti {
                    		y = x*y*2.0 + yy
                    		x = xsquared - ysquared + xx
                    		xsquared = x*x
                    		ysquared = y*y
                    		if xsquared+ysquared>i_limit {
                    			break }
                    		iter++
                		}

                	if iter != ti { 
                		myplot(pixelx, pixely, iter+16) 
                		if yz_flag == 1 and pixely != 119 myplot(pixelx, 239-pixely, iter+16) 
                		ti = max_iter }
                	else {
                		ti = max_iter*0.9 as ubyte }
                    	 }
                     i++
            		}
            		
            	i = 0
            	for pixelx in 40 to 79 {
            		if pixelx & 1 {
                		xx = index2[i]
					
                		xsquared = 0.0
                		ysquared = 0.0
                		x = 0.0
                		y = 0.0
                		iter = 0

                		while iter<ti {
                    		y = x*y*2.0 + yy
                    		x = xsquared - ysquared + xx
                    		xsquared = x*x
                    		ysquared = y*y
                    		if xsquared+ysquared>i_limit {
                    			break }
                    		iter++
                		}

                	if iter != ti { 
                		myplot(pixelx, pixely, iter+16) 
                		if yz_flag == 1 and pixely != 119 myplot(pixelx, 239-pixely, iter+16) 
                		ti = max_iter }
                		else {
                			ti = max_iter*0.9 as ubyte }
                    	 }
                    i++
                    	 }            
                    	 
                i = 0
            	for pixelx in 80 to 119 {
            		if pixelx & 1 {
                		xx = index3[i]
						
                		xsquared = 0.0
                		ysquared = 0.0
                		x = 0.0
                		y = 0.0
                		iter = 0

                		while iter<ti {
                    		y = x*y*2.0 + yy
                    		x = xsquared - ysquared + xx
                    		xsquared = x*x
                    		ysquared = y*y
                    		if xsquared+ysquared>i_limit {
                    			break }
                    		iter++
                		}

                	if iter != ti { 
                		myplot(pixelx, pixely, iter+16) 
                		if yz_flag == 1 and pixely != 119 myplot(pixelx, 239-pixely, iter+16)  
                		ti = max_iter }
                	else {
                		ti = max_iter*0.9 as ubyte }
                    	 }
                    i++
                    }                    
                    	 
                i = 0  	 
            	for pixelx in 120 to 159 {
            		if pixelx & 1 {
                		xx = index4[i]
						
                		xsquared = 0.0
                		ysquared = 0.0
                		x = 0.0
                		y = 0.0
                		iter = 0

                		while iter<ti {
                    		y = x*y*2.0 + yy
                    		x = xsquared - ysquared + xx
                    		xsquared = x*x
                    		ysquared = y*y
                    		if xsquared+ysquared>i_limit {
                    			break }
                    		iter++
                		}

                	if iter != ti { 
                		myplot(pixelx, pixely, iter+16) 
                		if yz_flag == 1 and pixely != 119 myplot(pixelx, 239-pixely, iter+16) 
                		ti = max_iter }
                	else {
                		ti = max_iter*0.9 as ubyte }
                    	 }
                	i++
                    }                    
                    
                i = 0
            	for pixelx in 160 to 199 {
            		if pixelx & 1 {
                		xx = index5[i]

                		xsquared = 0.0
                		ysquared = 0.0
                		x = 0.0
                		y = 0.0
                		iter = 0

                		while iter<ti {
                    		y = x*y*2.0 + yy
                    		x = xsquared - ysquared + xx
                    		xsquared = x*x
                    		ysquared = y*y
                    		if xsquared+ysquared>i_limit {
                    			break }
                    		iter++
                		}

                	if iter != ti { 
                		myplot(pixelx, pixely, iter+16) 
                		if yz_flag == 1 and pixely != 119 myplot(pixelx, 239-pixely, iter+16) 
                		ti = max_iter }
                	else {
                	 ti = max_iter*0.9 as ubyte }
                    	 }
                 i++
            }                    	 
            
            	i = 0
            	for pixelx in 200 to 239 {
            		if pixelx & 1 {
                		xx = index6[i]

                		xsquared = 0.0
                		ysquared = 0.0
                		x = 0.0
                		y = 0.0
                		iter = 0

                		while iter<ti {
                    		y = x*y*2.0 + yy
                    		x = xsquared - ysquared + xx
                    		xsquared = x*x
                    		ysquared = y*y
                    		if xsquared+ysquared>i_limit {
                    			break }
                    		iter++
                		}

                	if iter != ti { 
                		myplot(pixelx, pixely, iter+16) 
                		if yz_flag == 1 and pixely != 119 myplot(pixelx, 239-pixely, iter+16) 
                		ti = max_iter }
                	else {
                		ti = max_iter*0.9 as ubyte }
                    	 }
                   i++
            }            

				i = 0
            	for pixelx in 240 to 279 {
            		if pixelx & 1 {
                		xx = index7[i]

                		xsquared = 0.0
                		ysquared = 0.0
                		x = 0.0
                		y = 0.0
                		iter = 0

                		while iter<ti {
                    		y = x*y*2.0 + yy
                    		x = xsquared - ysquared + xx
                    		xsquared = x*x
                    		ysquared = y*y
                    		if xsquared+ysquared>i_limit {
                    			break }
                    		iter++
                		}

                	if iter != ti { 
                		myplot(pixelx, pixely, iter+16) 
                		if yz_flag == 1 and pixely != 119 myplot(pixelx, 239-pixely, iter+16)  
                		ti = max_iter }
                	else {
                		ti = max_iter*0.9 as ubyte }
                    	 }
                 i++
            }            

				i = 0
            	for pixelx in 280 to 319 {
            		if pixelx & 1 {
                		xx = index8[i]

                		xsquared = 0.0
                		ysquared = 0.0
                		x = 0.0
                		y = 0.0
                		iter = 0

                		while iter<ti {
                    		y = x*y*2.0 + yy
                    		x = xsquared - ysquared + xx
                    		xsquared = x*x
                    		ysquared = y*y
                    		if xsquared+ysquared>i_limit {
                    			break }
                    		iter++
                		}

                	if iter != ti { 
                		myplot(pixelx, pixely, iter+16) 
                		if yz_flag == 1 and pixely != 119 myplot(pixelx, 239-pixely, iter+16) 
                		ti = max_iter }
                	else {
                		ti = max_iter*0.9 as ubyte }
                    	 }
                   i++
            }
		  if yz_flag == 1 {
			if pixely == 119 break }
        }
 
;        time()
        

		cx = 159
		cy = 119
}

sub julia(float zoom, ubyte max_iter, float xpos, float ypos ) {
	float newRE
	float newIM
	float oldRE
	float oldIM
	float newIM2
	float newRE2
	float r
	float ty
	ubyte iter 
	ubyte i
	ubyte ti
	word y
	ubyte yz_flag
	ubyte px

	uword pixelx
	word pixely
	

				
			ti = max_iter
	
			;scale, per pixel, increment
			r = 1/my
			r *= (1/zm)
			r /= 10
			yz_flag = 0
			
			if ypos < 0 ty = -ypos
			else {
				ty = ypos }
	
			; flag for plot mirroring
			;if ypos < 1 pixel distance from zero, turn on mirroring
			
		  	if ypos == 0 yz_flag = 1
			else if ty < r {
				yz_flag = 1 }	
				
			;cbm.SETTIM(0, 0, 0)
			
			cx16.clock_set_date_time(0,0,0,0)
			
					
					
			;plot julia set, every other column to 'speed up' ;-)
			;120 == half height
			for pixely in 0 to height-1 {
            	y = (pixely - 120)
            	ty = y/(zoom*120) + ypos
            	i = 0
            	
            	for pixelx in 0 to 39 step 2 {

						
                		newRE = index1[i]
						newIM = ty					
						newRE2 = newRE * newRE
						newIM2 = newIM * newIM
                		iter = 0
                		px = 0
                		while iter<ti  {
            
							newIM = 2 * newRE * newIM + cIM
							newRE = newRE2 - newIM2 + cRE
							newRE2 = newRE * newRE
							newIM2 = newIM * newIM
                    		if newRE2 + newIM2 > i_limit {
                    			px = iter+16
                    			break }
                    		iter++
                		}
                		           			
        				rowbuf_a[pixelx as ubyte] = px
        				rowbuf_d[160-pixelx as ubyte] = px						              		               		             		
                		
                     i+=2
            	}
            		
            	i = 0
            	for pixelx in 40 to 79 step 2 {
				
                		newRE = index2[i]
						newIM = ty					
								
						newRE2 = newRE * newRE
						newIM2 = newIM * newIM
                		iter = 0
                		px = 0
                		while iter<ti  {
							newIM = 2 * newRE * newIM + cIM
							newRE = newRE2 - newIM2 + cRE
							newRE2 = newRE * newRE
							newIM2 = newIM * newIM
                    		if newRE2 + newIM2 > i_limit {
                    			px = iter+16
                    			break }
                    		iter++
                		}
                		
        				rowbuf_a[pixelx as ubyte] = px
        				rowbuf_d[160-pixelx as ubyte] = px        

                    i+=2
                 }            
                    	 
                i = 0
            	for pixelx in 80 to 119 step 2 {
                		newRE = index3[i]
						newIM = ty					
						newRE2 = newRE * newRE
						newIM2 = newIM * newIM
                		iter = 0
                		px = 0
                		while iter<ti  {
							newIM = 2 * newRE * newIM + cIM
							newRE = newRE2 - newIM2 + cRE
							newRE2 = newRE * newRE
							newIM2 = newIM * newIM
                    		if newRE2 + newIM2 > i_limit {
                    			px = iter+16
                    			break }
                    		iter++
                		}
                		    
        				rowbuf_a[pixelx as ubyte] = px
        				rowbuf_d[160-pixelx as ubyte] = px
                		
                    i+=2
                }                    
                    	 
                i = 0  	 
            	for pixelx in 120 to 159 step 2 {
                		newRE = index4[i]
						newIM = ty					
						newRE2 = newRE * newRE
						newIM2 = newIM * newIM
                		iter = 0
                		px = 0
                		while iter<ti  {
							newIM = 2 * newRE * newIM + cIM
							newRE = newRE2 - newIM2 + cRE
							newRE2 = newRE * newRE
							newIM2 = newIM * newIM
                    		if newRE2 + newIM2 > i_limit {
                    			px = iter+16
                    			break }
                    		iter++
                		}
                		                	
        				rowbuf_a[pixelx as ubyte] = px
        				rowbuf_d[160-pixelx as ubyte] = px
          
                	i+=2
                }                    
                    
                i = 0
            	for pixelx in 160 to 199 step 2 {
                		newRE = index5[i]
						newIM = ty					
						newRE2 = newRE * newRE
						newIM2 = newIM * newIM
                		iter = 0
                		px = 0
                		while iter<ti  {
							newIM = 2 * newRE * newIM + cIM
							newRE = newRE2 - newIM2 + cRE
							newRE2 = newRE * newRE
							newIM2 = newIM * newIM
                    		if newRE2 + newIM2 > i_limit {
                    			px = iter+16
                    			break }
                    		iter++
                		}
                		               
        				rowbuf_b[pixelx as ubyte-160] = px
        				rowbuf_c[320-pixelx as ubyte] = px                		             		

                 	i+=2
            	}                    	 
            
            	i = 0
            	for pixelx in 200 to 239 step 2 {
                		newRE = index6[i]
						newIM = ty					
						newRE2 = newRE * newRE
						newIM2 = newIM * newIM
                		iter = 0
                		px = 0
                		while iter<ti  {
							newIM = 2 * newRE * newIM + cIM
							newRE = newRE2 - newIM2 + cRE
							newRE2 = newRE * newRE
							newIM2 = newIM * newIM
                    		if newRE2 + newIM2 > i_limit {
                    			px = iter+16
                    			break }
                    		iter++
                		}
                		             	
        				rowbuf_b[pixelx as ubyte-160] = px
        				rowbuf_c[320-pixelx as ubyte] = px                		
                		
                   i+=2
                             
            	}            

				i = 0
            	for pixelx in 240 to 279 step 2 {
                		newRE = index7[i]
						newIM = ty					
						newRE2 = newRE * newRE
						newIM2 = newIM * newIM
                		iter = 0
                		px = 0
                		while iter<ti  {
							newIM = 2 * newRE * newIM + cIM
							newRE = newRE2 - newIM2 + cRE
							newRE2 = newRE * newRE
							newIM2 = newIM * newIM
                    		if newRE2 + newIM2 > i_limit {
                    			px = iter + 16
                    			break }
                    		iter++
                		}

        				rowbuf_b[pixelx as ubyte-160] = px
        				rowbuf_c[320-pixelx as ubyte] = px               		
                		
                 	i+=2
            	}            

				i = 0
            	for pixelx in 280 to 319 step 2 {
                		newRE = index8[i]
						newIM = ty					
						newRE2 = newRE * newRE
						newIM2 = newIM * newIM
                		iter = 0
                		px = 0
                		while iter<ti  {
							newIM = 2 * newRE * newIM + cIM
							newRE = newRE2 - newIM2 + cRE
							newRE2 = newRE * newRE
							newIM2 = newIM * newIM
                    		if newRE2 + newIM2 > i_limit {
                    			px = iter+16
                    			break }
                    		iter++
                		}
                		               	
        				rowbuf_b[pixelx as ubyte-160] = px
        				rowbuf_c[320-pixelx as ubyte] = px               		
                	
                   i+=2
            	}
            	cx16.FB_cursor_position(0, pixely as uword)	
            	cx16.FB_set_pixels(&rowbuf_a,320)
            	if yz_flag == 1 and pixely < 119 {
            		cx16.FB_cursor_position(0, 239-pixely as uword)	
            		cx16.FB_set_pixels(&rowbuf_c,320) }
            

			if ypos+xpos+pixely == 120 break
        }
        
        time()
		ding_dong()	        
		write_file(wfilenam)

		cx = 159
		cy = 119
}


sub getkey() {
	ubyte key = 0
	while key == 0 {
		key = cbm.GETIN() }
	}

sub write_file(str file) {
	ubyte row
	ubyte cnt
	
	wfilenam = file
	
	cbm.SETLFS(1, 8, 2)

	cbm.SETNAM(len(wfilenam), wfilenam)
	
	cbm.OPEN()
	
	cbm.CHKOUT(1)
	
		for row in 0 to 239 {
			cx16.FB_cursor_position(0, row)
        	cx16.FB_get_pixels(&rowbuf_a,320) 
        	
        	for cnt in 0 to 159 {
        		cbm.CHROUT(rowbuf_a[cnt]) }
        	
        	for cnt in 0 to 159 {
        		cbm.CHROUT(rowbuf_b[cnt]) }
        		
        	}
        	
        cbm.CLALL()
 }
 
 
sub check_file(str file) -> ubyte {
	ubyte st

	cbm.SETLFS(1, 8, 2)
	
	cbm.SETNAM(string.length(file), file)
		
	st = cbm.OPEN()
			
	cbm.CLALL()
	
	return st
	
	}
 
 sub read_mand() {
	ubyte row
	ubyte cnt
	ubyte lgth
		
	lgth = string.length(mandfnr)
	
	cbm.SETLFS(2, 8, 2)
	
	cbm.SETNAM(lgth, mandfnr)
	
	cbm.OPEN()
	
	cbm.CHKIN(2)
	
	for row in 0 to 239 {	
	
	 	for cnt in 0 to 159 {
        	rowbuf_a[cnt] = cbm.CHRIN() }
        	
       	for cnt in 0 to 159 {
        	rowbuf_b[cnt] = cbm.CHRIN() }
			
		cx16.FB_cursor_position(0, row)
        cx16.FB_set_pixels(&rowbuf_a,320) 
        		
        }
        
        cbm.CLALL()

 }
 
sub zero_buff(uword buf, ubyte ln) {
	ubyte i
	
	for i in 0 to ln-1 {
		buf[i] = 0 }
}
 
sub read_file(str file) {
	ubyte row
	ubyte cnt
	ubyte lgth
	
	rfilenam = file
	
	cbm.SETLFS(1, 8, 2)

	cbm.SETNAM(len(rfilenam), rfilenam)
	
	cbm.OPEN()
	
	cbm.CHKIN(1)
	
	for row in 0 to 239 {	
	
	 	for cnt in 0 to 159 {
        	rowbuf_a[cnt] = cbm.CHRIN() }
        	
       	for cnt in 0 to 159 {
        	rowbuf_b[cnt] = cbm.CHRIN() }
			
		cx16.FB_cursor_position(0, row)
        cx16.FB_set_pixels(&rowbuf_a,320)       		
        }
        	
        cbm.CLALL()
 }
 
;pre-calculate these values so we only calculate them once per set!		
sub do_indexx(float z, float xp) {
	uword x1
	ubyte y1
	
		y1= 0
		for x1 in 0 to 39 {
			index1[y1] = 1.5 * (x1 as float - 160)
			index1[y1] = index1[y1]/(z*160.0) + xp
			y1++ }
			
		y1 = 0
		for x1 in 40 to 79 {
			index2[y1] = 1.5 * (x1 as float - 160)
			index2[y1] = index2[y1]/(z*160.0) + xp
			y1++ }			
					
		y1 = 0
		for x1 in 80 to 119 {
			index3[y1] = 1.5 * (x1 as float - 160)
			index3[y1] = index3[y1]/(z*160.0) + xp
			y1++ }			
			
		y1= 0
		for x1 in 120 to 159 {
			index4[y1] = 1.5 * (x1 as float - 160)
			index4[y1] = index4[y1]/(z*160.0) + xp
			y1++ }				
			
		y1 = 0
		for x1 in 160 to 199 {
			index5[y1] = 1.5 * (x1 as float - 160)
			index5[y1] = index5[y1]/(z*160.0) + xp
			y1++ }			
				
		y1 = 0
		for x1 in 200 to 239 {
			index6[y1] = 1.5 * (x1 as float - 160)
			index6[y1] = index6[y1]/(z*160.0) + xp
			y1++ }					
							
		y1 = 0
		for x1 in 240 to 279 {
			index7[y1] = 1.5 * (x1 as float - 160)
			index7[y1] = index7[y1]/(z*160.0) + xp
			y1++ }								
											
		y1 = 0
		for x1 in 280 to 319 {
			index8[y1] = 1.5 * (x1 as float - 160)
			index8[y1] = index8[y1]/(z*160.0) + xp
			y1++ }												
															
}												

inline asmsub myplot(uword xp @R0, word yp @R1, ubyte c @A) clobbers(A, X, Y) {										
	%asm {{										
		pha								
		jsr cx16.FB_cursor_position							
		pla				
		jsr cx16.FB_set_pixel							
		}}							
		}							
												
        	 
sub time() {

		ubyte seconds
		ubyte minutes
		ubyte hours
		
		%asm{{
			jsr $ff50
		}}
		

		seconds = cx16.r2H
		minutes = cx16.r2L
		hours = cx16.r1H
    		
			
			if hours < 1 {	  					
        		txt.plot(0, 28)
        		txt.print("finished in ")
        		txt.print_ub(minutes) 
        		txt.print(" minutes and ") 
        		txt.print_ub(seconds)
        		txt.print(" seconds!")				
        		}
        	else {			
        		txt.plot(0, 28)
        		txt.print("finished in ")
        		txt.print_ub(hours) 
        		txt.print(" hours and ") 
        		txt.print_ub(minutes) 
        		txt.print(" minutes!") 
       		
        		}        		
				
	}	 
	
;get date inserted into filename strings
sub get_date() {
	str day = "  "
	str year = "    "
	str month = "  "
	str zero = "0"
	ubyte y
	ubyte m
	ubyte d
	
	void cx16.clock_get_date_time()
	y = lsb(cx16.r0)
	m = msb(cx16.r0)
	d = lsb(cx16.r1)
	
	txt.plot(0,27)
    conv.str_uw(1900 + y)
    year = conv.string_out
    sys.memcopy(&year,&wfilenam+3,4)
    sys.memcopy(&year,&rfilenam+3,4) 
    
        if m < 10 {
            conv.str_ub(m) 
            sys.memcopy(&zero,&month,1)
            sys.memcopy(&conv.string_out,&month+1,1)
            }
        else {
        	conv.str_ub(m) 
        	month = conv.string_out 
        	 }
        	
       	
		sys.memcopy(&month,&wfilenam+7,2)
   		sys.memcopy(&month,&rfilenam+7,2)  
		 
     	
        if d < 10 {
            conv.str_ub0(d) }
        else {
        	conv.str_ub(d)  }
        day = conv.string_out
		sys.memcopy(&day,&wfilenam+9,2)
		sys.memcopy(&day,&rfilenam+9,2)
		
		sys.memcopy(&depth+history,&wfilenam+12,1)
		sys.memcopy(&depth+history,&rfilenam+12,1)
		
    }

    ; voice 0-16
	; waveform - PULSE,TRIANGLE,NOISE
	; note - 
	; volume - 0-63(loudest)
	; duratoion - 0-255
sub beep(ubyte voice, ubyte waveform, uword note, ubyte volume, ubyte duration) {
		uword v			
		
	v = $F9C0 + voice

	cx16.vpoke(1,v+3,waveform)
	cx16.vpoke(1,v+2,volume)
	cx16.vpoke(1,v+1,msb(note))
	cx16.vpoke(1,v+0,lsb(note))
	sys.wait(duration)
	
	cx16.vpoke(1,v+2,0)
}

sub ding_dong() {
	beep(0,SQUARE,B,VOLUME,20)				
	beep(0,SQUARE,E,VOLUME,20)					
	beep(0,SQUARE,GSHARP,VOLUME,20)				
	beep(0,SQUARE,FSHARP,VOLUME,20)
}


sub def_sprite(ubyte bnk, uword sp_data, uword data_loc, ubyte sp_no, bool enable) {
	uword addr = $FC00+sp_no*8
	ubyte value
	ubyte en
	ubyte x
	uword bb			
	
	if enable en = $0c
	else en = $00
	
	bb = (data_loc / $10) | $1000
		
	;set vera location for sprite data	
	cx16.vaddr(bnk,data_loc,0,1)
	;copy sprite data to vera memory
	cx16.FB_set_pixels(sp_data,256)


	if bnk == 0 cx16.vpoke(1,addr,lsb(data_loc)>>5)
	else {	;if sprite is stored in bank 1
		value = bb>>5 as ubyte & $0f
		value = value * $10
		cx16.vpoke(1,addr,value) }

	addr++
	
	if bnk == 0 cx16.vpoke(1,addr,msb(data_loc)>>5 | $80)
	else {	;if sprite is stored in bank 1
		value = bb >> 5 as ubyte & $f0
		value = value / $10
		value = value | $80 
		cx16.vpoke(1,addr,value)}
	
	addr++
	value = 0
	for x in 0 to 3 {
		cx16.vpoke(1,addr,value)
		addr++ }
	cx16.vpoke(1,addr,en)
	addr++
	cx16.vpoke(1,addr,$50)
	
	}		
															
}



